from supervisely.nn.inference.inference import Inference
from supervisely.nn.inference.instance_segmentation.instance_segmentation import (
    InstanceSegmentation
)
from supervisely.nn.inference.object_detection.object_detection import (
    ObjectDetection
)



from typing import Dict, List

try:
    from typing import Literal
except ImportError:
    from typing_extensions import Literal

from supervisely.app.widgets import (
    Widget,
    SelectWorkspace,
    generate_id,
)
from supervisely.app import StateJson
from supervisely.api.api import Api
from supervisely.project.project_type import ProjectType
from supervisely.sly_logger import logger
from supervisely.app.widgets.select_sly_utils import _get_int_or_env


class SelectProject(Widget):
    def __init__(
        self,
        default_id: int = None,
        workspace_id: int = None,
        compact: bool = False,
        allowed_types: List[ProjectType] = [],
        show_label: bool = True,
        size: Literal["large", "small", "mini"] = None,
        widget_id: str = None,
    ):
        self._api = Api()
        self._default_id = default_id
        self._ws_id = workspace_id
        self._team_id = None
        self._compact = compact
        self._project_types = allowed_types
        self._show_label = show_label
        self._size = size
        self._ws_selector = None

        self._default_id = _get_int_or_env(self._default_id, "modal.state.slyProjectId")
        if self._default_id is not None:
            info = self._api.project.get_info_by_id(self._default_id, raise_error=True)
            self._ws_id = info.workspace_id
        self._ws_id = _get_int_or_env(self._ws_id, "context.workspaceId")

        if self._ws_id is not None:
            info = self._api.workspace.get_info_by_id(self._ws_id)
            self._team_id = info.team_id
        self._team_id = _get_int_or_env(self._team_id, "context.teamId")

        if compact is True:
            if self._ws_id is None:
                raise ValueError(
                    '"workspace_id" have to be passed as argument or "compact" has to be False'
                )
        else:
            # if self._show_label is False:
            #     logger.warn(
            #         "show_label can not be false if compact is True and default_id / workspace_id are not defined"
            #     )
            self._show_label = True
            self._ws_selector = SelectWorkspace(
                default_id=self._ws_id,
                team_id=self._team_id,
                compact=False,
                show_label=True,
                size=self._size,
                widget_id=generate_id(),
            )
        super().__init__(widget_id=widget_id, file_path=__file__)

    def get_json_data(self) -> Dict:
        res = {}
        res["workspaceId"] = self._ws_id
        res["options"] = {
            "showLabel": self._show_label,
            "compact": self._compact,
            "filterable": True,
            "availableTypes": [ptype.value for ptype in self._project_types],
        }
        if self._size is not None:
            res["options"]["size"] = self._size
        return res

    def get_json_state(self) -> Dict:
        return {
            "projectId": self._default_id,
        }

    def get_selected_id(self):
        return StateJson()[self.widget_id]["projectId"]



# coding: utf-8

import json
from supervisely.api.module_api import ApiField
from supervisely.video_annotation.key_id_map import KeyIdMap
from supervisely.volume_annotation.volume_annotation import VolumeAnnotation

from supervisely.api.entity_annotation.entity_annotation_api import EntityAnnotationAPI
from supervisely.io.json import load_json_file


class VolumeAnnotationAPI(EntityAnnotationAPI):
    _method_download_bulk = "volumes.annotations.bulk.info"
    _entity_ids_str = ApiField.VOLUME_IDS

    def download(self, volume_id):
        """
        :param video_id: int
        :return: video annotation to given id in json format
        """
        volume_info = self._api.volume.get_info_by_id(volume_id)
        return self._download(volume_info.dataset_id, volume_id)

    def append(self, volume_id, ann: VolumeAnnotation, key_id_map: KeyIdMap = None):
        info = self._api.volume.get_info_by_id(volume_id)
        self._append(
            self._api.volume.tag,
            self._api.volume.object,
            self._api.volume.figure,
            info.project_id,
            info.dataset_id,
            volume_id,
            ann.tags,
            ann.objects,
            ann.figures,
            key_id_map,
        )

    def upload_paths(
        self,
        volume_ids,
        ann_paths,
        project_meta,
        interpolation_dirs=None,
        progress_cb=None,
    ):
        if interpolation_dirs is None:
            interpolation_dirs = [None] * len(ann_paths)

        key_id_map = KeyIdMap()
        for volume_id, ann_path, interpolation_dir in zip(
            volume_ids, ann_paths, interpolation_dirs
        ):
            ann_json = load_json_file(ann_path)
            ann = VolumeAnnotation.from_json(ann_json, project_meta)
            self.append(volume_id, ann, key_id_map)

            # create empty figures for meshes
            self._api.volume.figure.append_bulk(
                volume_id, ann.spatial_figures, key_id_map
            )
            # upload existing interpolations or create on the fly and and add them to empty mesh figures
            self._api.volume.figure.upload_stl_meshes(
                volume_id, ann.spatial_figures, key_id_map, interpolation_dir
            )
            if progress_cb is not None:
                progress_cb(1)



# coding: utf-8

from supervisely.video_annotation.video_tag_collection import VideoTagCollection
from supervisely.volume_annotation.volume_tag import VolumeTag


class VolumeTagCollection(VideoTagCollection):
    item_type = VolumeTag



from typing import Union, List
from supervisely.app import DataJson
from supervisely.app.widgets import (
    Widget,
    ObjectClassView,
    Checkbox,
    Grid,
    generate_id,
    Button,
)
from supervisely.annotation.obj_class import ObjClass
from supervisely.annotation.obj_class_collection import ObjClassCollection


"""
from supervisely.geometry.bitmap import Bitmap
from supervisely.geometry.cuboid import Cuboid
from supervisely.geometry.point import Point
from supervisely.geometry.polygon import Polygon
from supervisely.geometry.polyline import Polyline
from supervisely.geometry.rectangle import Rectangle
from supervisely.geometry.graph import GraphNodes
from supervisely.geometry.any_geometry import AnyGeometry
from supervisely.geometry.cuboid_3d import Cuboid3d
from supervisely.geometry.pointcloud import Pointcloud
from supervisely.geometry.point_3d import Point3d
from supervisely.geometry.multichannel_bitmap import MultichannelBitmap
from supervisely.geometry.closed_surface_mesh import ClosedSurfaceMesh

_classes = [
    ObjClass("person1", AnyGeometry),
    ObjClass("person2", Rectangle),
    ObjClass("person3", Polygon),
    ObjClass("person4", Bitmap),
    ObjClass("person5", Point),
    ObjClass("person6", Polyline),
    ObjClass("person7", Cuboid),
    ObjClass("person8", Point3d),
    ObjClass("person9", Cuboid3d),
    ObjClass("person10", Pointcloud),
    ObjClass("person11", Cuboid3d),
    ObjClass("person12", Pointcloud),
    ObjClass("person13", ClosedSurfaceMesh),
    ObjClass("person14", MultichannelBitmap),
]

model_classes = ObjectClassesList(object_classes=_classes, columns=3, selectable=True)

text = Text("123")
classes_card = Card(
    "Model classes",
    "Model predicts the following classes",
    content=model_classes,
)
"""


class ObjectClassesList(Widget):
    def __init__(
        self,
        object_classes: Union[ObjClassCollection, List[ObjClass]],
        selectable: bool = False,
        columns: int = 1,  # 1 means vertical layout
        widget_id: str = None,
    ):
        self._object_classes = object_classes
        self._selectable = selectable
        self._columns = columns

        if type(object_classes) is list:
            self._object_classes = ObjClassCollection(self._object_classes)

        self._name_to_class = {}
        self._name_to_view = {}
        self._name_to_checkbox = {}
        self._select_all_btn = None
        self._deselect_all_btn = None
        for obj_class in self._object_classes:
            self._name_to_class[obj_class.name] = obj_class
            self._name_to_view[obj_class.name] = ObjectClassView(
                obj_class, widget_id=generate_id()
            )
            grid_items = list(self._name_to_view.values())
            if self._selectable is True:
                self._select_all_btn = Button(
                    "Select all",
                    button_type="text",
                    show_loading=False,
                    icon="zmdi zmdi-check-all",
                    widget_id=generate_id(),
                )
                self._deselect_all_btn = Button(
                    "Deselect all",
                    button_type="text",
                    show_loading=False,
                    icon="zmdi zmdi-square-o",
                    widget_id=generate_id(),
                )
                current_checkbox = Checkbox(
                    self._name_to_view[obj_class.name],
                    checked=True,
                    widget_id=generate_id(),
                )
                self._name_to_checkbox[obj_class.name] = current_checkbox
                grid_items = list(self._name_to_checkbox.values())

                @self._select_all_btn.click
                def select_all():
                    for k, v in self._name_to_checkbox.items():
                        v: Checkbox
                        v.check()
                    DataJson()[self.widget_id]["num_selected"] = len(
                        self._name_to_checkbox
                    )
                    DataJson().send_changes()

                @self._deselect_all_btn.click
                def deselect_all():
                    for k, v in self._name_to_checkbox.items():
                        v: Checkbox
                        v.uncheck()
                    DataJson()[self.widget_id]["num_selected"] = 0
                    DataJson().send_changes()

                @current_checkbox.value_changed
                def checkbox_changed(is_checked):
                    DataJson()[self.widget_id]["num_selected"] = len(
                        self.get_selected_classes()
                    )
                    DataJson().send_changes()

        self._content = Grid(
            widgets=grid_items,
            columns=self._columns,
            widget_id=generate_id(),
        )
        super().__init__(widget_id=widget_id, file_path=__file__)

    def get_json_data(self):
        res = {"selectable": self._selectable}
        if self._selectable is True:
            res["num_selected"] = len(self.get_selected_classes())
        return res

    def get_selected_classes(self):
        if self._selectable is False:
            raise ValueError(
                "Class selection is disabled, because ObjectClassesList widget was created with argument 'selectable == False'"
            )
        results = []
        for k, v in self._name_to_checkbox.items():
            v: Checkbox
            if v.is_checked():
                results.append(k)
        return results

    def get_json_state(self):
        return None
